from importlib import resources
from pathlib import Path
import logging
import logging.config
import traceback
import sys
import yaml


PGM_DIR = Path("collectlicense")
APP_ID = 'collectlicense'
CMD = None

def load_config():
    global CMD
    logconf_str = resources.read_text('collectlicense','logconf.yml')
    logging.config.dictConfig(yaml.safe_load(logconf_str))
    logger = logging.getLogger('main')
    config_str = resources.read_text('collectlicense','config.yml')
    config = yaml.safe_load(config_str)
    CMD = config['collectlicense']['common']['CMD']
    return config, logger

def mkdirs(dir_path:Path):
    if not dir_path.exists():
        dir_path.mkdir(parents=True)
    if not dir_path.is_dir():
        raise BaseException(f"Don't make diredtory.({str(dir_path)})")
    return dir_path

def e_msg(e:Exception, logger):
    tb = sys.exc_info()[2]
    logger.error(traceback.format_exc())
    return e.with_traceback(tb)




from collectlicense.app import common
from pathlib import Path
import json
import re
import shutil
import subprocess

def safe_fname(fname:str) -> str:
    """
    ファイル名に使えない文字を置換します。

    Args:
        fname (str): ファイル名

    Returns:
        str: 置換後のファイル名
    """
    #fname = re.sub('[_]', '-_-',str(fname))
    fname = re.sub('[\s:;\\\\/,\.\?\#\$\%\^\&\!\@\*\~\|\<\>\(\)\{\}\[\]\'\"\`]', '_',str(fname))
    fname = re.sub('[_]{2,}', '-', fname)
    return fname.strip('_').strip('-').strip()

def main(out_dir:Path, clear:bool):
    _, logger = common.load_config()
    if clear:
        shutil.rmtree(out_dir, ignore_errors=True)
    common.mkdirs(out_dir)
    proc = subprocess.run(common.CMD, capture_output=True, text=True)
    if proc.returncode != 0:
        logger.error(proc.stderr, stack_info=True)
        return
    output_str = proc.stdout
    license_json = json.loads(output_str)
    exclude_key = ('LicenseText','LicenseFile')
    maxfnsize = 128
    with open(out_dir / f"files.txt", "w", encoding="utf-8") as lf:
        for i, license_info in enumerate(license_json):
            ln = license_info['License'].translate(str.maketrans({'\\':'','/':'',':':'','*':'','?':'','"':'','<':'','>':'','|':''}))
            fn = safe_fname(f"LICENSE.{license_info['Name']}.{license_info['Version']}({ln})") + '.txt'
            if len(fn) > maxfnsize:
                fn = fn[:maxfnsize] + '.txt'
            output_file = out_dir / fn
            license_info['File'] = str(output_file)
            cols = [key for key in license_info if key not in exclude_key]
            if i==0:
                lf.write('\t'.join(cols)+'\n')
            with open(output_file, "w", encoding="UTF-8") as f:
                f.write(license_info['LicenseText'])
            lf.write('\t'.join([license_info[col] for col in cols if col not in exclude_key])+'\n')




from pathlib import Path
import argparse


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        prog='python -m collectlicense',
        description='Collect license files for packages installed with pip.')
    
    parser.add_argument('--out',
                        help='License filr output directory.',
                        default=Path(".") / ".licenses")
    parser.add_argument('--clear',
                        help='Clear output directory.',
                        action='store_true')

    args = parser.parse_args()
    from collectlicense.app import app
    app.main(Path(args.out), args.clear)



from collectlicense import version
from setuptools import setup


DESCRIPTION = 'collect-license: Collect license files for packages installed with pip.'
NAME = 'collectlicense'
AUTHOR = 'hamacom2004jp'
AUTHOR_EMAIL = 'hamacom2004jp@gmail.com'
URL = 'https://github.com/hamacom2004jp/collect-license'
LICENSE = 'MIT'
DOWNLOAD_URL = URL
VERSION = version.__version__
PYTHON_REQUIRES = '>=3.8'
INSTALL_REQUIRES = [
    'PyYAML>=6.0.1',
    'pip-licenses>=4.3.3'
]
PACKAGES = [
    'collectlicense',
    'collectlicense.app'
]
KEYWORDS = 'LICENSE.txt LICENSE Collect'
CLASSIFIERS=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Intended Audience :: Information Technology',
    'Intended Audience :: System Administrators',
    'License :: OSI Approved :: MIT License',
    'Natural Language :: Japanese',
    'Programming Language :: Python',
    'Topic :: Utilities'
]
with open('README.md', 'r', encoding='utf-8') as fp:
    readme = fp.read()
LONG_DESCRIPTION = readme
LONG_DESCRIPTION_CONTENT_TYPE = 'text/markdown'
RESORCE_TEXT_FILES = dict(collectlicense=['config.yml','logconf.yml'])

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESCRIPTION_CONTENT_TYPE,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    maintainer=AUTHOR,
    maintainer_email=AUTHOR_EMAIL,
    url=URL,
    download_url=URL,
    packages=PACKAGES,
    classifiers=CLASSIFIERS,
    license=LICENSE,
    keywords=KEYWORDS,
    install_requires=INSTALL_REQUIRES,
    package_data=RESORCE_TEXT_FILES,
    include_package_data=True
)